/*
Copyright (c) 2009, Dan "Ducky" Little & GeoMOOSE.org

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/* these, theoretically, could be broken out into separate class
	files, but for now, this seems to be sufficient */


GeoMOOSE.Services = {};

/*
 * InputType
 * Base class for all of the GeoMOOSE inputs types used with services.
 */
GeoMOOSE.Services.InputType = OpenLayers.Class({
	/*
	 * The value of the input-type
	 */
	value: null,

	/* XML input */
	input: null,

	initialize: function(input, options) {
		this.input = input;

		this.options = {
			renderable : false /* set whether this needs rendered in a form */
		};
		OpenLayers.Util.extend(this.options, options || {});		

		/* set the input based on the "value" attribute */
		if(input.getAttribute('value')) {
			this.value = input.getAttribute('value');
		}

		/* set the input value based on the value of the childnodes */
		if(input.childNodes.length > 0) {
			this.value = OpenLayers.Util.getXmlNodeValue(input);
		}
	},

	/* requiresRender()
	 * return whether or not to render this input.
	 */
	requiresRender: function() {
		return this.options.renderable;
	},

	/*
	 * setValue()
	 */
	setValue: function(value) {
		this.value = value;

		var node = this.input;
		/* clear the current children */
		if(node.childNodes) {
			while(node.childNodes.length > 0) {
				node.removeChild(node.childNodes[0]);
			}
		}
	
		/* create a new CDATA section with the value */
		node.appendChild(node.ownerDocument.createCDATASection(this.getValue()));
	},

	/*
	 * getValue()
	 * Gets the value of the input
	 */
	getValue: function() {
		/* This does no escaping, and essentially just lets things roll...*/
		if(!this.value) {
			return '';
		}
		return this.value;
	},

	/*
	 * getName()
	 * get the name of hte input
	 */
	getName: function() {
		return this.input.getAttribute('name');
	},

	/*
	 * renderHTML(parent_id)
	 * Render the form representation of the input type in the parent specified
	 * by parent_id.
	 */
	renderHTML: function (parent_id) {
	},

	MAPBOOK_NAME: null
});

GeoMOOSE.Services.InputType.Hidden = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "hidden"
});

GeoMOOSE.Services.InputType.Extent = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "extent",

	getValue: function() {
		var delim = this.input.getAttribute('delim');
		if(!delim) { delim = ','; }
		return Map.getExtent().toArray().join(delim);
	}
});

GeoMOOSE.Services.InputType.VisibleLayers = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "visiblelayers",

	getValue: function() {
		var delim = this.input.getAttribute('delim');
		if(!delim) { delim = ':'; }
		return GeoMOOSE.getVisibleLayers().join(delim);
	}
});

GeoMOOSE.Services.InputType.Sketches = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "sketches",

	getValue: function() {
		var sketch_layer = Map.getLayersByName('sketchlayer')[0];
		var wkt = new OpenLayers.Format.WKT();

		var feature_info = [];
		for(var i = 0; i < sketch_layer.features.length; i++) {
			var feature = sketch_layer.features[i];

			var feature_obj = {};
			feature_obj['wkt'] = wkt.write(feature);
			for(var k in feature.attributes) {
				feature_obj[k] = feature.attributes[k];
			}

			feature_info.push(feature_obj);
		}

		var json = new OpenLayers.Format.JSON();
		return json.write(feature_info);
	}

});

GeoMOOSE.Services.InputType.PrintInfo = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "print_info",

	getValue: function() {
		//input.appendChild(Catalog.createCDATASection(getPrintRequest()));
		var info = getPrintRequest(); /* returns an object with all the inputs needed for a print */
		var vis_layers = GeoMOOSE.getVisibleLayers();
		var services = {};
	
		var print_obj = {};
		/* specify the layer printing order */
		var service_order = [];
		var found_service = {}
		for(var i = 0; i < vis_layers.length; i++) {
			var service_name = vis_layers[i].split('/')[0];
			if(!found_service[service_name]) {
				service_order.push(service_name);
			}
			found_service[service_name] = true;
		}
		print_obj['order'] = service_order;

		for(var i = 0; i < vis_layers.length; i++) {				
			var path_arr = vis_layers[i].split('/');
			if(!print_obj[path_arr[0]]) {
				print_obj[path_arr[0]] = {};
				print_obj[path_arr[0]]['layers'] = [];
				print_obj[path_arr[0]]['params'] = {};
				print_obj[path_arr[0]]['opacity'] = GeoMOOSE.getLayerOpacity(path_arr[0]);
				print_obj[path_arr[0]]['url'] = GeoMOOSE.getLayerUrl(path_arr[0]); 
			}
			print_obj[path_arr[0]]['layers'].push(path_arr[1]);

			var params = GeoMOOSE.getLayerParameters(path_arr[0]);
			for(var p in params) {
				print_obj[path_arr[0]]['params'][p] = params[p];
			}
		}

		var json = new OpenLayers.Format.JSON();
		return json.write(print_obj);
	}
});
		
GeoMOOSE.Services.InputType.User = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "user",

	initialize: function(input, options) {
		GeoMOOSE.Services.InputType.prototype.initialize.apply(this, arguments);
		this.options.renderable = true;
	},

	updateServiceSetting: function() {
		//console.log(this.value);
		this.input_obj.setValue(this.value);
	},

	keyCallService: function(event) {
		if(window.event) { event = window.event; }
		var keyCode = null;
		if(event.which) {
			keyCode = event.which;
		} else if(event.keyCode) {
			keyCode = event.keyCode;
		}
		if(keyCode == 13) {
			this.onchange();
			document.getElementById('submit-service').onclick();
		}
	},

	renderHTML: function(parent_id) {
		var p = document.getElementById(parent_id);
		var inputParent = document.createElement('span');
		inputParent.className = 'service-input-parent';
		p.appendChild(inputParent);

		var i = document.createElement('input');
		i.className = 'service-input';
		i.input_obj = this;
		i.id = 'service-input-'+this.getName();

		if(this.getValue()) {
			i.value = this.getValue();
		}
		i.onchange = this.updateServiceSetting;
		i.onkeypress = this.keyCallService;

		var title = document.createElement('span');
		title.className = 'service-input-title';
		inputParent.appendChild(title);
		title.innerHTML = this.input.getAttribute('title');
		inputParent.appendChild(i);
	}
	
});

GeoMOOSE.Services.InputType.Select = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "select",
	input: null,

	initialize: function(input, options) {
		this.options = {
			renderable: true
		};
		this.input = input;
	},

	getOptions: function() {
		var opts = this.input.getElementsByTagName('option');
		var formed_opts = [];
		for(var i = 0 ; i < opts.length; i++) {
			formed_opts.push({value: opts[i].getAttribute('value'), name: OpenLayers.Util.getXmlNodeValue(opts[i])});
		}
		return formed_opts;
	},

	getValue: function() {
		return this.input.getAttribute('value');
	},

	setValue: function(v) {
		this.input.setAttribute('value', v);
	},

	updateSettings: function() {
		this.input_obj.setValue(this.getElementsByTagName('option')[this.selectedIndex].value);
	},

	renderHTML: function(parent_id) {
		var p = document.getElementById(parent_id);
		var inputParent = document.createElement('span');
		inputParent.className = 'service-input-parent';
		p.appendChild(inputParent);

		var select = document.createElement('select');
		select.input_obj = this;

		var title = document.createElement('span');
		title.className = 'service-input-title';
		inputParent.appendChild(title);
		title.innerHTML = this.input.getAttribute('title');
		inputParent.appendChild(select);

		var options = this.getOptions();
		var selIndex = 0;
		var val = this.getValue();
		for(var i = 0; i < options.length; i++) {
			if(options[i].value == val) {
				selIndex = i;
			}

			var opt = document.createElement('option');
			opt.selected= false;
			opt.setAttribute('value', options[i].value);
			select.appendChild(opt);
			opt.appendChild(document.createTextNode(options[i].name));
		}

		var selected_option = select.getElementsByTagName('option')[selIndex];
		selected_option.selected = true;
		this.setValue(selected_option.value);
			
		select.onchange = this.updateSettings;
	}

});

/*
 * Get the options from a URL
 * The returned XML should follow a convenction of HTML-style options, e.g.
 * <options>
 *	<option value="a">A</option>
 *	<option value="b">B</option>
 * </options>
 */
GeoMOOSE.Services.InputType.AjaxSelect = OpenLayers.Class(GeoMOOSE.Services.Select, {
	MAPBOOK_NAME: "ajax_select",

	getOptions: function () {
		var formed_opts = [];
		var url = this.input.getElementsByTagName('url')[0];

		url = OpenLayers.Util.getXmlNodeValue(url);

		var requestOptions = OpenLayers.Request.GET({
		       url: url,
		       async: false
		});

		var response = requestOptions.responseXML;
		var opts = response.getElementsByTagName('option'); 
		for(var i = 0 ; i < opts.length; i++) {
		       formed_opts.push({value: opts[i].getAttribute('value'), name: OpenLayers.Util.getXmlNodeValue(opts[i])});
		}
		return formed_opts;     
	}
});

GeoMOOSE.Services.InputType.Tolerance = OpenLayers.Class(GeoMOOSE.Services.InputType, {
	MAPBOOK_NAME: "tolerance",

	getValue: function() {
		return Map.getResolution() * 10;
	}
});

